// ignore_for_file: non_constant_identifier_names, camel_case_extensions

import 'dart:async';
import 'package:galaxeus_lib/extension/regexp.dart';
import 'package:telegram_client/telegram_client/scheme/json_dart.dart';
import 'package:telegram_client/telegram_client/telegram_client.dart';
import 'package:telegram_client/util/util.dart';

/// method SendMessage
extension SendMessageRawDataOn on TelegramClient {
  /// method SendMessageRaw
  FutureOr<JsonScheme> sendMessageRaw({
    required Map parameters,
    required FutureOr<JsonScheme> Function({
      required Map parameters,
    }) callApiInvoke,
    required TelegramClientData telegramClientData,
  }) async {
    dynamic target_chat_id = TgUtils.parse_all_chat_id(parameters: parameters);
    if (target_chat_id is String &&
        RegExp(r"^((@)[a-z0-9_]+)$", caseSensitive: false)
            .hashData(target_chat_id)) {
      var search_public_chat = await callApiInvoke(
        parameters: {
          "@type": "searchPublicChat",
          "username": (target_chat_id)
              .replaceAll(RegExp(r"@", caseSensitive: false), ""),
        },
      );
      if (search_public_chat["@type"] == "chat") {
        parameters["chat_id"] = search_public_chat["id"];
      }
    }
    Map request_parameters = {
      "@type": "sendMessage",
      "chat_id": parameters["chat_id"],
      "input_message_content": {},
    };
    if (parameters.containsKey("message_thread_id")) {
      request_parameters["message_thread_id"] = parameters["message_thread_id"];
    }
    if (RegExp(r"^(sendMessage)$", caseSensitive: false)
        .hashData(parameters["@type"])) {
      Map text_data = tdlib.td_execute(
        {
          "@type": 'parseTextEntities',
          "parse_mode": {
            "@type": "textParseModeHTML",
          },
          "text": parameters["text"],
        },
      );

      request_parameters["input_message_content"]["@type"] = "inputMessageText";
      request_parameters["input_message_content"]["text"] = text_data;
    }
    JsonScheme message_send = await callApiInvoke(
      parameters: request_parameters,
    );
    Map result = {};

    var listen = on(
      event_name: event_update,
      onUpdate: (UpdateTelegramClient updateTelegramClient) async {
        if (updateTelegramClient.telegramClientData.telegramClientType ==
            TelegramClientType.tdlib) {
          if (updateTelegramClient.telegramClientData.tdlib_client_id !=
              telegramClientData.tdlib_client_id) {
            return;
          }
          Map updateOrigin = updateTelegramClient.rawData;
          if (!["updateMessageSendSucceeded", "updateMessageSendFailed"]
              .contains(updateOrigin["@type"])) {
            return;
          }

          if (updateOrigin["message"] is Map &&
              updateOrigin["message"]["chat_id"] == message_send["chat_id"] &&
              updateOrigin["old_message_id"] == message_send["id"]) {
            result = updateOrigin;
          }
        }
      },
      onError: (Object error, StackTrace stackTrace) {},
    );

    while (true) {
      await Future.delayed(Duration(microseconds: 1));
      if (result["@type"] is String) {
        event_emitter.off(listen);

        if (result["@type"] == "error") {
          throw result;
        }
        result.remove("@type");
        if (result["message"] is Map) {
          Map json_message = await message_Message(
              message: result["message"],
              telegramClientData: telegramClientData);

          return JsonScheme(json_message);
        }
      }
    }
  }
}
